@@ -1,11 +1,12 @@
 from typing import NamedTuple
 from datetime import timedelta
+from google.protobuf.message import Message
 from .messages_pb2 import TaskEntry, GroupEntry, PipelineEntry, TaskRetryPolicy, Pipeline
 from ._utils import _type_name
 import json
 
 
-class _TaskEntry(object):
+class Task(object):
     def __init__(self, task_id, task_type, args, kwargs, parameters=None, is_finally=False):
         self.task_id = task_id
         self.task_type = task_type
@@ -29,7 +30,7 @@ class _TaskEntry(object):
         return f'{self.get_parameter("namespace")}/{self.get_parameter("worker_name")}'
 
     def to_tuple(self):
-        return self.task_id, self.task_type, self.args, self.kwargs
+        return self.task_id, self.task_type, self.args, self.kwargs, self.parameters
 
     def mark_complete(self):
         self.complete = True
@@ -51,11 +52,8 @@ class _TaskEntry(object):
             complete=self.complete, 
             is_finally=self.is_finally)
         
-        if self.args is not None:
-            proto.args.CopyFrom(serialiser.to_proto(self.args))
-
-        if self.kwargs is not None: 
-            proto.kwargs.CopyFrom(serialiser.to_proto(self.kwargs))
+        request = serialiser.serialise_args_and_kwargs(self.args, self.kwargs)
+        proto.request.CopyFrom(request)
 
         if self.parameters is not None:
             proto.parameters.CopyFrom(serialiser.to_proto(self.parameters))
@@ -64,11 +62,13 @@ class _TaskEntry(object):
 
     @staticmethod 
     def from_proto(proto: PipelineEntry, serialiser):
-        entry = _TaskEntry(
+        args, kwargs = serialiser.deserialise_args_and_kwargs(proto.task_entry.request)
+
+        entry = Task(
             task_id=proto.task_entry.task_id, 
             task_type=proto.task_entry.task_type, 
-            args=serialiser.from_proto(proto.task_entry.args), 
-            kwargs=serialiser.from_proto(proto.task_entry.kwargs), 
+            args=args, 
+            kwargs=kwargs, 
             parameters=serialiser.from_proto(proto.task_entry.parameters), 
             is_finally=proto.task_entry.is_finally)
 
@@ -80,7 +80,7 @@ class _TaskEntry(object):
         return self.task_id
 
 
-class _GroupEntry(object):
+class Group(object):
     def __init__(self, group_id):
         self.group_id = group_id
         self._group = []
@@ -91,6 +91,9 @@ class _GroupEntry(object):
     def get_parameter(self, parameter_name):
         return None
 
+    def get_destination(self):
+        return None  # _GroupEntries don't have a single destination
+
     def add_to_group(self, tasks):
         self._group.append(tasks)
 
@@ -124,7 +127,7 @@ class _GroupEntry(object):
 
     @staticmethod 
     def from_proto(proto: PipelineEntry, serialiser):
-        entry = _GroupEntry(group_id=proto.group_entry.group_id)
+        entry = Group(group_id=proto.group_entry.group_id)
 
         group = []
         for pipeline in proto.group_entry.group:
@@ -132,9 +135,9 @@ class _GroupEntry(object):
             
             for proto in pipeline.entries:
                 if proto.HasField('task_entry'):
-                    entries.append(_TaskEntry.from_proto(proto, serialiser))
+                    entries.append(Task.from_proto(proto, serialiser))
                 elif proto.HasField('group_entry'):
-                    entries.append(_GroupEntry.from_proto(proto, serialiser))
+                    entries.append(Group.from_proto(proto, serialiser))
 
             group.append(entries)
 
@@ -157,3 +160,8 @@ class RetryPolicy(NamedTuple):
             max_retries=self.max_retries, 
             delay_ms=self.delay.total_seconds() * 1000, 
             exponential_back_off=self.exponential_back_off)
+
+
+class TaskAlreadyExistsException(Exception):
+    def __init__(self, message):
+        super().__init__(message)
